import os
import json
import pandas as pd
from report.entity.domain_config import DomainConfig
from report.elasticsearch_rule_converter import *
import report

def generate_report(customer_name,stam_name,review_date,data_folder,output_folder):
    if os.path.exists(output_folder):
        print("WARN: output folder exist")
        exit(0)
    else:
        os.mkdir(output_folder)

    report_file=output_folder + "/elasticsearch_report.md"
    print("generating report in: " + report_file)

    if os.path.exists(data_folder):
        print("generating report from the data in the folder: " + data_folder)
    else:
        print("data folder does not exist: " + data_folder)
        exit(1)

    gen_summary(customer_name, stam_name, review_date,report_file)
    gen_domain_config(data_folder, report_file)
    gen_cluster_settings(data_folder, report_file)
    gen_cluster_health(data_folder, report_file)
    gen_cluster_state(data_folder, report_file)
    gen_cluster_stats(data_folder, report_file)
    gen_cluster_nodes(data_folder, report_file)
    gen_index(data_folder, report_file)
    gen_shard(data_folder, report_file)
    gen_allocation(data_folder, report_file)
    #gen_field(data_folder, report_file)
    #gen_mapping(data_folder, report_file)
    gen_thread_pool(data_folder, report_file)
    gen_recommendation(data_folder, report_file)
    gen_appendix(data_folder, report_file)
    #gen_reference(report_file)

def write_line(report_file,line):
    with open(report_file,"a+", encoding='utf-8') as f:
        f.write(line + "\n")

def gen_summary(customer_name,stam_name,review_date,report_file):
    write_line(report_file,"## Elasticsearch Review for " + customer_name)
    write_line(report_file,"### Elasticsearch Specialist -- Enterprise Support")
    write_line(report_file,"Enterprise Support Specalist: " + stam_name)
    write_line(report_file,"Review Date: " +review_date)
    write_line(report_file, "")


def gen_domain_config(data_folder,report_file):
    write_line(report_file, "## Elasticsearch Cluster - Infrastructure and Security:")
    write_line(report_file, "Based on the information from ES Domain Config, we provide the following recommendations:")
    write_line(report_file, "")
    with open(data_folder + "/domain_config.txt") as f:
        config_txt = f.read()
        config = eval(config_txt)
        #print("ES version:" + str(config['ElasticsearchVersion']))
        version = config["ElasticsearchVersion"]["Options"]
        it = config["ElasticsearchClusterConfig"]["Options"]["InstanceType"]
        ic = config["ElasticsearchClusterConfig"]["Options"]["InstanceCount"]
        dme = config["ElasticsearchClusterConfig"]["Options"]["DedicatedMasterEnabled"]
        dmt = ""
        if "DedicatedMasterType" in config["ElasticsearchClusterConfig"]["Options"]:
            dmt = config["ElasticsearchClusterConfig"]["Options"]["DedicatedMasterType"]
        zae = config["ElasticsearchClusterConfig"]["Options"]["ZoneAwarenessEnabled"]
        ebs = config["EBSOptions"]["Options"]["VolumeType"]
        aei = config["AdvancedOptions"]["Options"]["rest.action.multi.allow_explicit_index"]
        ear = config["EncryptionAtRestOptions"]["Options"]["Enabled"]
        ne = config["NodeToNodeEncryptionOptions"]["Options"]["Enabled"]
        https = config["DomainEndpointOptions"]["Options"]["EnforceHTTPS"]
        fgac = config["AdvancedSecurityOptions"]["Options"]["Enabled"]
        internal_user_database = config["AdvancedSecurityOptions"]["Options"]["InternalUserDatabaseEnabled"]
        vpc = str(config["VPCOptions"]["Options"])
        access_policy = str(config["AccessPolicies"]["Options"])
        log_publish = str(config["LogPublishingOptions"]["Options"])
        warm_enabled=False
        if "WarmEnabled" in config["ElasticsearchClusterConfig"]["Options"]:
            warm_enabled = config["ElasticsearchClusterConfig"]["Options"]["WarmEnabled"]
        domain_config = DomainConfig(version, it, ic, dme, dmt, zae, ebs, aei, ear, ne, https,fgac,internal_user_database,vpc,access_policy,log_publish,warm_enabled)
        rec1 = get_recommendation("domain_config.txt",domain_config)
        for r in rec1:
            rec_text = rec1[r]
            if(rec_text.upper() != "NO RECOMMENDATION"):
                write_line(report_file, "* **RECOMMENDATION** " + rec1[r])

def gen_rec_from_rule(report_file,metadata_file, data):
    rec1 = get_recommendation_from_data(metadata_file, data)
    # print("recommendations are: " + str(rec1))
    for r in rec1:
        rec_text = rec1[r]["recommendation"]
        field = rec1[r]["field"]
        original = rec1[r]["original"]
        if (original != ""):
            write_line(report_file, "*  " + field + " : " + str(original))
        if (rec_text.upper() != "NO RECOMMENDATION"):
            write_line(report_file, "   * **RECOMMENDATION** " + rec_text)


def gen_cluster_settings(data_folder,report_file):
    write_line(report_file, "## Cluster Settings Details")
    # TODO parse json file and gen report
    write_line(report_file, "")
    with open(data_folder + "/cluster_settings_default.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file, "cluster_settings_default.json", data)


def gen_cluster_health(data_folder,report_file):
    write_line(report_file, "## Cluster Health")
    write_line(report_file, "")
    with open(data_folder + "/cluster_health.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file,"cluster_health.json",data)

def gen_cluster_state(data_folder,report_file):
    write_line(report_file, "## Cluster State")
    write_line(report_file, "")
    with open(data_folder + "/cluster_state.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file,"cluster_state.json",data)

def gen_cluster_stats(data_folder,report_file):
    write_line(report_file, "")
    write_line(report_file, "## Cluster Stats")
    write_line(report_file, "")
    with open(data_folder + "/cluster_stats.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file, "cluster_stats.json", data)

def gen_thread_pool(data_folder,report_file):
    write_line(report_file, "## Thread Pool")
    write_line(report_file, "")
    with open(data_folder + "/thread_pool_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_rec_from_rule(report_file, "thread_pool_table.csv", data)
    write_line(report_file, "See the appendix in [ThreadPool](#ThreadPool) for more detailed information.")

def gen_cluster_nodes(data_folder,report_file):
    write_line(report_file, "## Cluster Nodes Details")
    write_line(report_file, "")

    #write_line(report_file, "### Master Nodes")
    #write_line(report_file, "")
    #with open(data_folder + "/master.json") as f:
    #    data = json.load(f)
    #    gen_rec_from_rule(report_file, "master.json", data)

    #write_line(report_file, "### Data Nodes")
    #write_line(report_file, "")
    #with open(data_folder + "/nodes.json") as f:
    #    data = json.load(f)
    #    gen_rec_from_rule(report_file, "nodes.json", data)

    write_line(report_file, "## Data Nodes")
    write_line(report_file, "")
    with open(data_folder + "/nodes_cat_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_rec_from_rule(report_file, "nodes_cat_table.csv", data)
    write_line(report_file, "See the appendix in [DataNodes](#DataNodes) for more detailed information.")

    write_line(report_file, "###  Nodes stats")
    write_line(report_file, "")
    with open(data_folder + "/nodes_stats.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file, "nodes_stats.json", data)

    write_line(report_file, "###  Nodes usage:")
    write_line(report_file, "")
    with open(data_folder + "/nodes_usage.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file, "nodes_usage.json", data)


def gen_index(data_folder,report_file):
    write_line(report_file, "## Index")
    write_line(report_file, "")
    with open(data_folder + "/indices_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_rec_from_rule(report_file, "indices_table.csv", data)
    write_line(report_file, "See the appendix in [Index](#Index) for more detailed information.")

def gen_field(data_folder,report_file):
    write_line(report_file, "")
    write_line(report_file, "## Fields")
    write_line(report_file, "Fields information is as following:")
    # TODO parse json file and gen report
    write_line(report_file, "")
    with open(data_folder + "/fielddata.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file, "fielddata.json", data)


def gen_mapping(data_folder,report_file):
    write_line(report_file, "")
    write_line(report_file, "## Mappings")
    write_line(report_file, "Mappings information is as following:")
    # TODO parse json file and gen report
    write_line(report_file, "")
    with open(data_folder + "/mapping.json") as f:
        data = json.load(f)
        gen_rec_from_rule(report_file, "mapping.json", data)

def gen_allocation(data_folder,report_file):
    write_line(report_file, "## Allocation")
    write_line(report_file, "")
    with open(data_folder + "/allocation_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_rec_from_rule(report_file, "allocation_table.csv", data)
    write_line(report_file, "See the appendix in [Allocation](#Allocation) for more detailed information.")

def gen_shard(data_folder,report_file):
    write_line(report_file, "## Shards")
    write_line(report_file, "")
    with open(data_folder + "/shards_detail_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_rec_from_rule(report_file, "shards_detail_table.csv", data)
    write_line(report_file, "See the appendix in [Shards](#Shards) for more detailed information.")


def gen_recommendation(data_folder,report_file):
    write_line(report_file, "")
    config_file = os.path.join(report.__path__[0], "general_recommendation.md")
    with open(config_file) as f:
        rec = f.read()
        write_line(report_file,rec)

def gen_appendix(data_folder,report_file):
    write_line(report_file, "## Appendix")

    write_line(report_file, "### DataNodes")
    with open(data_folder + "/nodes_cat_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_table_appendix(report_file, data)

    write_line(report_file, "### Index")
    with open(data_folder + "/indices_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_table_appendix(report_file, data)

    write_line(report_file, "### Shards")
    with open(data_folder + "/shards_detail_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_table_appendix(report_file, data)

    write_line(report_file, "### Allocation")
    with open(data_folder + "/allocation_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_table_appendix(report_file, data)

    write_line(report_file, "### ThreadPool")
    with open(data_folder + "/thread_pool_table.csv") as f:
        data = pd.read_csv(f, delim_whitespace=True)
        gen_table_appendix(report_file, data)



def gen_table_appendix(report_file, df):
    header = "|"
    for c in df.columns:
        header += c
        header += "|"
    write_line(report_file,header)

    header = "|"
    for c in df.columns:
        header += "--|"
    write_line(report_file,header)

    for index, row in df.iterrows():
        row_text = "|"
        for c in df.columns:
            row_text += str(row[c])
            row_text += "|"
        write_line(report_file,row_text)

def gen_reference(report_file):
    write_line(report_file, "## Reference")
    write_line(report_file, "ref1")
    write_line(report_file, "")
