## Elasticsearch Index and Shards
* **Bulk Requests:** The Bulk API makes it possible to perform many index/delete operations in a single API call. This can greatly increase the indexing speed. Each subrequest is executed independently, so the failure of one subrequest won’t affect the success of the others. If any of the requests fail, the top-level error flag is set to true and the error details will be reported under the relevant request.
* **Multithread clients to Index Data:** A single thread sending bulk requests is unlikely to be able to max out the indexing capacity of an Elasticsearch cluster. In order to use all resources of the cluster, you should send data from multiple threads or processes. In addition to making better use of the resources of the cluster, this should help reduce the cost of each fsync. Both the index data and transaction log are periodically flushed to disk. If there are more data with multithread, the more data is synced to disk to reduce I/O to improve performance.
* **index.refresh_interval:** By default, Elasticsearch periodically refreshes indices every second, but only on indices that have received one search request or more in the last 30 seconds.This is the optimal configuration if you have no or very little search traffic (e.g. less than one search request every 5 minutes) and want to optimize for indexing speed. This behavior aims to automatically optimize bulk indexing in the default case when no searches are performed. In order to opt out of this behavior set the refresh interval explicitly.On the other hand, if your index experiences regular search requests, this default behavior means that Elasticsearch will refresh your index every 1 second. If you can afford to increase the amount of time between when a document gets indexed and when it becomes visible, increasing the index.refresh_interval to a larger value, e.g. 30s, might help improve indexing speed.
* **Auto generated IDs:** When indexing a document that has an explicit id, Elasticsearch needs to check whether a document with the same id already exists within the same shard, which is a costly operation and gets even more costly as the index grows. By using auto-generated ids, Elasticsearch can skip this check, which makes indexing faster.
* **index.translog.sync_interval:** This parameter determines how often the translog is **fsynced** to disk and committed, regardless of write operations. Defaults to 5s. Values less than 100ms are not allowed.
* **index.translog.flush_threshold_size**: The translog stores all operations that are not yet safely persisted in Lucene (i.e., are not part of a Lucene commit point). Although these operations are available for reads, they will need to be replayed if the shard was stopped and had to be recovered. This setting controls the maximum total size of these operations, to prevent recoveries from taking too long. Once the maximum size has been reached a flush will happen, generating a new Lucene commit point. Defaults to 512mb.
* **Large Documents:** Large documents put more stress on network, memory usage and disk. Indexing large document can use an amount of memory that is a multiplier of the original size of the document. Proximity search (phrase queries for instance) and highlighting also becomes more expensive since their cost directly depends on the size of the original document.
* **Set Index Mapping Explicitly:** Elasticsearch can create mapping dynamically, but it might be not suitable for all scenarios. For example, the default string field mappings in Elasticsearch 5.x are both "keyword" and "text" types. It's unnecessary in a lot of scenarios.
* **Index Mapping - Nested Types:** Querying on nested fields is slower compared to fields in parent document. Retrieval of matching nested fields adds an additional slowdown. Once you update any field of a document containing nested fields, independent of whether you updated a nested field or not, all the underlying Lucene documents (parent and all its nested children) need to be marked as deleted and rewritten. In addition to slowing down your updates, such an operation also creates garbage to be cleaned up by segment merging later on.
* **Index Mapping:**  Disable the **_all** [field](https://www.elastic.co/guide/en/elasticsearch/reference/5.4/mapping-all-field.html)  concatenates the values of all other fields into one string. It requires more CPU and disk space than other fields. Most use cases don't require the _all field. You can concatenate multiple fields using the [copy_to](https://www.elastic.co/guide/en/elasticsearch/reference/5.4/mapping-all-field.html#custom-all-fields) parameter. The _all field is disabled by default in Elasticsearch versions 6.0 and later. To disable the _all field in earlier versions, set enabled to false.
* **Leverage Index Templates:** Index templates define settings like number of shards, replicas and mappings that you can automatically apply when creating new indices. Elasticsearch applies templates to new indices based on an index pattern that matches the index name.
* **Use Replicas for Scalability & Resilience:** Elasticsearch is built to be always available and to scale with your needs. It does this by being distributed in nature. You can add nodes to a cluster to increase capacity and Elasticsearch automatically distributes your data and query load across all of the available nodes. For ElasticSearch to be highly available, its indices needs to have fault tolerance in place. This can be achived using replica shards. A replica shard is a copy of a primary shard. Replicas provide redundant copies of your data to protect against hardware failure and increase capacity to serve read requests like searching or retrieving a document.
* **Shard Sizing:** A shard is a Lucene index under the covers, which uses file handles, memory, and CPU cycles. Default shard strategy for an index in ES is 5 primary shards with a replica. The goal of choosing a number of shards is to distribute an index evenly across all data nodes in the cluster. However, these shards shouldn't be too large or too numerous. A good rule of thumb is to try to keep shard size between 10–50 GB. Large shards can make it difficult for Elasticsearch to recover from failure, but because each shard uses some amount of CPU and memory, having too many small shards can cause performance issues and out of memory errors. In other words, shards should be small enough that the underlying Amazon ES instance can handle them, but not so small that they place needless strain on the hardware.
* **Keep shard count of index in multiple of data nodes**  with **equivalent size** and **distributed across nodes**. Set a primary shard count, via a template, by targeting 50GB max per primary shard (log analytics) or 30GB max (search use cases)
  Shard assignment across data nodes will happen based on 2 important rules.
  * **Primary** and **replica** shard of same index will not be assigned on same data nodes. 
  * A shard is placed on a node based on how many shards are available on that node or to equalize the number of shards per index across all nodes in the cluster. Also note Amazon Elasticsearch master will never take consideration of how much disk is consumed on any data node before placing the shard. So there can be chances of bigger shards allocated to some node and smaller to others. It is recommended that shard count of an index (primary + replica) should be multiple of data node count. Let's say, you have a 4 node cluster, the total shards (primary + Replica) for your index should be either 4 or 8 or 12 etc. This ensures that the data is evenly distributed across the nodes.
* **Index State Management:** ISM lets you define custom management policies to automate routine tasks and apply them to indices and index patterns. You no longer need to set up and manage external processes to run your index operations.A policy contains a default state and a list of states for the index to transition between. Within each state, you can define a list of actions to perform and conditions that trigger these transitions. A typical use case is to periodically delete old indices after a certain period of time.

* **Organize the data in index by date:** For most logging or monitoring use cases, we can organize indices to be daily, weekly, or monthly, and then we can get an index list by a specified date range. Elasticsearch only needs to query on a smaller dataset instead of the whole dataset. In addition, it would be easy to shrink/delete the old indices when data has expired.

* **Use Curator to rotate data:** Curator offers numerous filters to help you identify indices and snapshots that meet certain criteria, such as indices created more than 60 days ago or snapshots that failed to complete

## Elasticsearch Query & Search
* **Filter context instead of query context(if applicable):** A query clause is used to answer “How well does this document match this clause?”. A filter clause is used to answer “Does this document match this clause?” Elasticsearch only needs to answer “Yes” or “No.” It does not need to calculate a relevancy score for a filter clause, and the filter results can be cached.
* **Index refresh interval:** Elasticsearch creates new segment every time a refresh happens. Increasing the refresh interval would help reduce the segment count and reduce the IO cost for search. And, the cache would be invalid once a refresh happens and data is changed. Increasing the refresh interval can make Elasticsearch utilize cache more efficiently.
* **Increase replica count:** Elasticsearch can perform a search on either a primary or replica shard. The more replicas you have, the more nodes can be involved in your search.
* **Retrieve only necessary fields:** If your documents are large, and you need only a few fields, use stored_fields to retrieve fields you need instead of all fields.
* **Wildcard Queries:** Try to avoid query patterns with wildcards such as * or ?. This can increase the iterations needed to find matching terms and slow the search performance.
* **Read only index (force merge):** Indices that are read-only would benefit from being merged down to a single segment. This is typically the case with time-based indices: only the index for the current time frame is getting new documents while older indices are read-only. The force merge API allows to force merging of one or more indices through an API. The merge relates to the number of segments a Lucene index holds within each shard. The force merge operation allows to reduce the number of segments by merging them. This call will block until the merge is complete. If the http connection is lost, the request will continue in the background, and any new requests will block until the previous force merge is complete.
* **Leverage Node Query Cache:** The results of queries used in the filter context are cached in the node query cache for fast lookup. There is one queries cache per node that is shared by all shards. The cache uses an LRU eviction policy: when the cache is full, the least recently used query results are evicted to make way for new data. You cannot inspect the contents of the query cache.
* **Shard Query Cache:** The shard-level request cache module caches the local results on each shard. This allows frequently used (and potentially heavy) search requests to return results almost instantly. The requests cache is a very good fit for the logging use case, where only the most recent index is being actively updated — results from older indices will be served directly from the cache.

## Monitoring
* It is recommended to configure the key metrics & alarms documented below, more details about these metrics can be referenced [here](https://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/cloudwatch-alarms.html). 

| **Name**                            | **Metric** | **Threshold**           | **Periods** |
| ----------------------------------- | ---------- | ----------------------- | ----------- |
| ClusterStatus.red                   | Maximum    | >= 1                    | 1           |
| ClusterIndexWritesBlocked           | Maximum    | >= 1                    | 1           |
| CPUUtilization/MasterCPUUtilization | Average    | >= 80%                  | 3           |
| JVMMemoryPressure/Master...         | Maximum    | >= 80%                  | 3           |
| FreeStorageSpace                    | Minimum    | <= (25% of avail space) | 1           |
| AutomatedSnapshotFailure            | Maximum    | >= 1                    | 1           |
| Queue depth                         | Average    | >= 100                  | 1           |
| Rejected Executions                 | Maximum    | >= 1                    | 1           |

## Cost Optimization
* By starting with an understanding of your storage needs, the service, and Elasticsearch itself, you can make an educated initial estimate on your hardware needs. This estimate can serve as a useful starting point for the most critical aspect of sizing domains: testing them with representative workloads and monitoring their performance. For more details on **Calculating the storage requirements**, **Choosing the number of shards** and **Choosing the instance types** please refer [here](https://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/sizing-domains.html).

